searchNodes=[{"ref":"Genex.html","title":"Genex","type":"behaviour","doc":"Genetic Algorithms in Elixir! Genex is a simple library for creating Genetic Algorithms in Elixir. A Genetic Algorithm is a search-based optimization technique based on the principles of Genetics and Natural Selection. The basic life-cycle of a Genetic Algorithm is as follows: 1) Initialize the Population 2) Loop until goal is reached a) Select Parents b) Perform Crossover c) Mutate some of population d) Select Survivors Genex follows a structure similar to the one above and offers callbacks corresponding to each stage in the genetic algorithm to allow for full customization. Implementation Genex requires an implementation module: defmodule OneMax do use Genex def encoding do for _ &lt;- 1..15, do: Enum.random(0..1) end def fitness_function(chromosome), do: Enum.sum(chromosome.genes) def terminate?(population), do: population.max_fitness == 15 end Genex requires 3 function definitions: encoding/0, fitness_function/1, and terminate?/1. Let&#39;s take a closer look at each of these: Encoding def encoding do for _ &lt;- 1..15, do: Enum.random(0..1) end encoding/0 defines your encoding of the chromosome&#39;s genes for your use-case. In the example above, we define a Binary Gene Set of length 15. Genex uses this function to generate an initial population of Chromosomes matching your encoding. Fitness Function def fitness_function(chromosome) do Enum.sum(chromosome.genes) end fitness_function/1 defines how the algorithm evaluates the fitness of a chromosome. It takes in a chromosome struct and returns a number. Fitness is how your algorithm determines which chromosomes should be persisted to the next generation as well as which chromosomes should be selected to crossover. In this case, we want to maximize 1&#39;s in our set of genes, so we define fitness as the sum of genes. Termination Criteria def terminate?(population) do population.max_fitness == 15 end terminate?/1 defines the termination criteria for your algorithm. This tells Genex when your algorithm should stop running. In this case we use a Max Fitness; however, you can also tell the algorithm to stop after a certain number of generations. Running Once you have defined an implementation module. Utilize the run/0 function to run the algorithm. The function will return the solution population for analysis. You can display a summary of the solution with the: Genex.Visualizers.Text.display_summary/1 function. soln = OneMax.run() Genex.Visualizers.Text.display_summary(soln) Configuration Genex offers a number of settings to adjust the algorithm to your liking. You can adjust: strategies, rates, and more. Below is a comprehensive list of settings and options. Strategies :crossover_type- :single_point, :two_point, :uniform, :blend :mutation_type- :scramble, :invert, :bit_flip :parent_selection- :natural, :random, :worst :survivor_selection- :natural, :random, :worst Rates :crossover_rate- between 0 and 1 :mutation_rate- between 0 and 1 Population :population_size- integer greater than 0 Unique to some Strategies :uniform_crossover_rate- between 0 and 1 (Uniform Crossover) :alpha- between 0 and 1 (Blend Crossover) Customization You can customize every step of your genetic algorithm utilizing some of the many callbacks Genex provides. A list of callbacks is provided below. seed/0 - Seed the Population. evaluate/1 - Evaluate the entire Population. cycle/1 - The Genetic Algorithm Cycle. select_parents/1 - Select Parents for Crossover. crossover/1 - Perform Crossover. mutate/1 - Perform Mutation. select_survivors/1 - Select a number of chromosomes to survive. advance/1 - Advance to the next generation."},{"ref":"Genex.html#c:crossover/1","title":"Genex.crossover/1","type":"callback","doc":"Crossover a number of individuals to create a new population. The number of individuals depends on the crossover rate. This phase populates the children field of the populaton struct with a List of Chromosomes."},{"ref":"Genex.html#c:encoding/0","title":"Genex.encoding/0","type":"callback","doc":"Generates a random gene set."},{"ref":"Genex.html#c:evaluate/1","title":"Genex.evaluate/1","type":"callback","doc":"Evaluates a population&#39;s fitness."},{"ref":"Genex.html#c:fitness_function/1","title":"Genex.fitness_function/1","type":"callback","doc":"Calculates a Chromosome&#39;s fitness."},{"ref":"Genex.html#c:mutate/1","title":"Genex.mutate/1","type":"callback","doc":"Mutate a number of individuals to add novelty to the population. The number of individuals depends on the mutation rate. This phase populates the mutant field of the population struct with a List of Chromosomes."},{"ref":"Genex.html#c:seed/0","title":"Genex.seed/0","type":"callback","doc":"Seeds a population."},{"ref":"Genex.html#c:select_parents/1","title":"Genex.select_parents/1","type":"callback","doc":"Selects a number of individuals for crossover. The number of individuals selected depends on the crossover rate. This phase populates the parent field of the population struct with a List of tuples. Each tuple is a pair of parents to crossover."},{"ref":"Genex.html#c:select_survivors/1","title":"Genex.select_survivors/1","type":"callback","doc":"Select a number of individuals to survive to the next generation. The number of individuals depends on the survival rate. This phase populates the survivors field of the population struct with a List of Chromosomes."},{"ref":"Genex.html#c:terminate?/1","title":"Genex.terminate?/1","type":"callback","doc":"Tests the population for some termination criteria."},{"ref":"Genex.Chromosome.html","title":"Genex.Chromosome","type":"module","doc":"A collection of genes. A Chromosome represents one solution to the problem you are trying to solve. Solutions are encoded into an enumerable of genes. The Chromosome is then evaluated based on some criteria you define."},{"ref":"Genex.Chromosome.html#get_fitness/1","title":"Genex.Chromosome.get_fitness/1","type":"function","doc":"Return the fitness of chromosome."},{"ref":"Genex.Chromosome.html#t:t/0","title":"Genex.Chromosome.t/0","type":"type","doc":""},{"ref":"Genex.Operators.Crossover.html","title":"Genex.Operators.Crossover","type":"module","doc":"Implementation of several popular crossover methods. Crossover is analagous to reproduction or biological crossover. Genex utilizes pairs of chromosomes to create offspring from the genetic material of parent chromosomes. Crossover happens with some probability P(c). Typically this is a high probability. The probability of crossover or crossover_rate as it is called in our case, determines the number of parents selected to breed for the next generation. See more on this in the Selection documentation. Crossover operators are generic. As with any optimization problem, no single method will be perfect. Genex offers a variety of crossover operators to experiment with; however, you may find that you need to write your own to fit your specific use case. You can do this by overriding the crossover method. Each time a crossover takes place, 2 new children are created. These children then populate the children field of the Population struct before they are merged into the new population."},{"ref":"Genex.Operators.Crossover.html#blend/3","title":"Genex.Operators.Crossover.blend/3","type":"function","doc":"Performs a blend crossover. This will blend genes according to some alpha between 0 and 1. If alpha=.5, the resulting chromosomes will be identical to one another. Returns Chromosome. Parameters p1: Parent one. p2: Parent two. alpha: Float between 0 and 1 representing percentage of each parent to blend into children."},{"ref":"Genex.Operators.Crossover.html#single_point/2","title":"Genex.Operators.Crossover.single_point/2","type":"function","doc":"Performs single point crossover. This will swap a slice of genes from each chromosome, producing 2 new chromosomes. Returns %Chromosome{}. Parameters p1: Parent One. p2: Parent Two."},{"ref":"Genex.Operators.Crossover.html#two_point/2","title":"Genex.Operators.Crossover.two_point/2","type":"function","doc":"Performs two-point crossover. This will swap multiple slices of genes from each chromosome, producing 2 new chromosomes. Returns %Chromosome{}. Parameters p1: Parent One. p2: Parent Two."},{"ref":"Genex.Operators.Crossover.html#uniform/3","title":"Genex.Operators.Crossover.uniform/3","type":"function","doc":"Performs uniform crossover. This will swap random genes from each chromosome according to some specified rate, producing 2 new chrmosomes. Returns Chromosome. Parameters p1: Parent One. p2: Parent Two. rate: Float between 0 and 1 representing rates to swap genes."},{"ref":"Genex.Operators.Mutation.html","title":"Genex.Operators.Mutation","type":"module","doc":"Implementation of several population mutation methods. Mutation takes place according to some rate. Mutation is useful for introducing novelty into the population. This ensures your solutions don&#39;t prematurely converge. Future versions of Genex will provide the ability to define the &quot;aggressiveness&quot; of mutations. As of this version of Genex, mutations effect the ENTIRE chromosome."},{"ref":"Genex.Operators.Mutation.html#bit_flip/1","title":"Genex.Operators.Mutation.bit_flip/1","type":"function","doc":"Perform a bit-flip mutation. This mutation performs a binary XOR on every gene in the Chromosome. Returns Chromosome. Parameters chromosome- Chromosome to mutate."},{"ref":"Genex.Operators.Mutation.html#invert/1","title":"Genex.Operators.Mutation.invert/1","type":"function","doc":"Perform inversion mutation. This mutation reverses (inverts) the genes of the Chromosome. Returns Chromosome. Parameters chromosome- Chromosome to mutate."},{"ref":"Genex.Operators.Mutation.html#scramble/1","title":"Genex.Operators.Mutation.scramble/1","type":"function","doc":"Perform a scramble mutation. This mutation shuffles the genes of the Chromosome. Returns Chromosome. Parameters chromosome- Chromosome to mutate."},{"ref":"Genex.Operators.Selection.html","title":"Genex.Operators.Selection","type":"module","doc":"Implementation of several popular selection methods. Selection occurs in two stages in Genex: parent selection and survivor selection. Parent Selection dictates which chromosomes are to be reserved for crossover according to some crossover rate. In this stage, a number of chromosomes are selected and paired off in 2-tuples in the order they are selected. Future versions of Genex will provide more advanced methods of parent selection. Survivor Selection occurs last in the GA cycle. As of this version of Genex, the survivor rate is always equal to 1 - CR where CR is the crossover rate. Future versions will support more advanced survivor selection, including the ability to fluctuate the population according to some operators."},{"ref":"Genex.Operators.Selection.html#natural/1","title":"Genex.Operators.Selection.natural/1","type":"function","doc":""},{"ref":"Genex.Operators.Selection.html#natural/2","title":"Genex.Operators.Selection.natural/2","type":"function","doc":"Natural selection of some number of chromosomes. This will select the n best (fittest) chromosomes. Returns Population. Parameters population: Population struct. type: :parents or :survivors. rate: Float representing survival rate or crossover rate."},{"ref":"Genex.Operators.Selection.html#random/1","title":"Genex.Operators.Selection.random/1","type":"function","doc":""},{"ref":"Genex.Operators.Selection.html#random/2","title":"Genex.Operators.Selection.random/2","type":"function","doc":"Random selection of some number of chromosomes. This will select n random chromosomes. Returns Population. Parameters population: Population struct. type: :parents or :survivors. rate: Float representing survival rate or crossover rate."},{"ref":"Genex.Operators.Selection.html#worst/1","title":"Genex.Operators.Selection.worst/1","type":"function","doc":""},{"ref":"Genex.Operators.Selection.html#worst/2","title":"Genex.Operators.Selection.worst/2","type":"function","doc":"Worst selection of some number of chromosomes. This will select the n worst (least fit) chromosomes. Returns Population. Parameters population: Population struct. type: :parents or :survivors. rate: Float representing survival rate or crossover rate."},{"ref":"Genex.Population.html","title":"Genex.Population","type":"module","doc":"A collection of Chromosomes. The Population represents the pool of solutions you are trying to optimize. Ideally, you persist the &#39;fittest&#39; features from generation to generation until you converge on a solution. The Population struct contains a number of useful fields that can be analyzed after the algorithm runs."},{"ref":"Genex.Population.html#sort/1","title":"Genex.Population.sort/1","type":"function","doc":"Sort the population by fitness."},{"ref":"Genex.Population.html#t:t/0","title":"Genex.Population.t/0","type":"type","doc":""},{"ref":"Genex.Support.Genealogy.html","title":"Genex.Support.Genealogy","type":"module","doc":"Implementation of a genealogy tree. We use the Genealogy tree to model the history of the population from it&#39;s initialization to the end of the algorithm. The tree itself is an erlang digraph. An edge emanates from parent and is incident on child."},{"ref":"Genex.Support.Genealogy.html#init/0","title":"Genex.Support.Genealogy.init/0","type":"function","doc":"Initializes a new Genealogy Tree. Returns Graph."},{"ref":"Genex.Support.Genealogy.html#update/2","title":"Genex.Support.Genealogy.update/2","type":"function","doc":"Updates a Genealogy Tree with just a vertex. Returns Graph. Parameters genealogy - Reference to a Genealogy Tree. chromosome - Chromosome to add to Genealogy."},{"ref":"Genex.Support.Genealogy.html#update/4","title":"Genex.Support.Genealogy.update/4","type":"function","doc":"Updates a Genealogy Tree with a vertex and it&#39;s corresponding parents. Returns Graph. Parameters genealogy - Reference to a Genealogy Tree. chromosome - Chromosome to add to genealogy. parent_a - Parent Chromosome. parent_b - Parent Chromosome."},{"ref":"Genex.Visualizers.Text.html","title":"Genex.Visualizers.Text","type":"module","doc":"Text visualizations of the Algorithm."},{"ref":"Genex.Visualizers.Text.html#display_summary/1","title":"Genex.Visualizers.Text.display_summary/1","type":"function","doc":"Display a summary of the population."},{"ref":"Genex.Visualizers.Text.html#init/0","title":"Genex.Visualizers.Text.init/0","type":"function","doc":"Display Initial Algorithm Text."}]